import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import uuid from 'uuid-v4';

export default class DisplayEnumComponent extends PureComponent {
    static propTypes = {
        clazzName: PropTypes.string,
        currentValue: PropTypes.string,
        label: PropTypes.string.isRequired,
        listOfValues: PropTypes.array.isRequired,
        name: PropTypes.string.isRequired,
        onInputChange: PropTypes.func.isRequired
    };

    render() {
        const {
            clazzName, label, listOfValues, name, onInputChange, currentValue
        } = this.props;

        return <div className={clazzName}>
            <span className="label">{label}</span>
            <span className="radioButtonGroup">
                {R.map((listItem) => {
                    const id = uuid();
                    return [
                        <input
                            checked={R.equals(listItem, currentValue)}
                            id={id}
                            key={`input-key-${id}`}
                            name={name}
                            onChange={onInputChange}
                            type="radio"
                            value={listItem}
                        />,
                        <label htmlFor={id} key={`label-key-${id}`}>{listItem}</label>
                    ];
                }, listOfValues)}
            </span>
        </div>;
    }
}
