import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {filterOutHiddenKeys} from '../../utils/filters';
import {localeMessages} from '../../services/locale-service';
import {createExtraBindingSections} from './playground-utils';
import ElementNavTitle from './element-nav-title';

export default class ShowTestComponent extends Component {
    static propTypes = {
        children: PropTypes.element,
        className: PropTypes.string,
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        updateState: PropTypes.func.isRequired
    };

    handleHideComponent = () => {
        this.updateComponentVisibility(false);
    };

    handleShowComponent = () => {
        this.updateComponentVisibility(true);
    };

    updateComponentVisibility = (isShown) => {
        const {currentState, modelName, updateState} = this.props;
        const mapping = currentState[modelName].$mapping;
        updateState(modelName, {
            ...currentState[modelName],
            [mapping.showPropertyName]: isShown
        });
    };

    render() {
        const {
            className,
            children,
            currentState,
            modelName,
            updateState
        } = this.props;

        const model = currentState[modelName];
        const {navigationDescription, navigationTitle, type} = model;

        const componentProperties = filterOutHiddenKeys(model);

        const hideFunctionParams = R.reduce((acc, functionName) => ({
            [functionName]: this.handleHideComponent,
            ...acc
        }), {}, R.pathOr([], ['$mapping', 'handleHideFunctionNames'], model));

        const newParams = {
            [model.$mapping.showPropertyName]: currentState[modelName][model.$mapping.showPropertyName],
            ...hideFunctionParams,
            ...componentProperties
        };

        const childrenWithProps = React.Children.map(children,
            (child) => React.cloneElement(child, newParams)
        );

        const onModelChange = (val) => {
            updateState(modelName, {...currentState[modelName], model: val});
        };

        const updateProps = (changedParams) => {
            updateState(modelName, {...currentState[modelName], ...changedParams});
        };

        const displayBindings = () => <div className="testBindings">
            <div>
                <span className="label">{localeMessages.apiLabel}</span>
                <button
                    className="xl-react-button xl-primary api-show-btn"
                    onClick={this.handleShowComponent}
                    type="button">
                    {localeMessages.showButton}
                </button>
            </div>
            {createExtraBindingSections(model.extraBindingSections, model.model, onModelChange, updateProps)}
        </div>;

        return <div>
            <ElementNavTitle
                description={navigationDescription}
                modelName={modelName}
                title={navigationTitle}
                type={type}
            />
            <div className={className}>
                <div className="testComponent">
                    {childrenWithProps}
                </div>
                {displayBindings()}
            </div>
        </div>;
    }
}
