import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import DisplayComponent from '../sink/display-component';
import RU from '../../utils/ramda-utils';
import ElementNavTitle from './element-nav-title';
import {createExtraBindingSections} from './playground-utils';

export default class SimpleTestComponent extends PureComponent {
    static propTypes = {
        children: PropTypes.oneOfType([
            PropTypes.element,
            PropTypes.array
        ]),
        className: PropTypes.string.isRequired,
        currentState: PropTypes.object.isRequired,
        displayedProperties: PropTypes.arrayOf(PropTypes.string),
        modelName: PropTypes.string.isRequired,
        updateState: PropTypes.func.isRequired
    };

    static defaultProps = {displayedProperties: []};

    displayItem = (componentState) => (prop, key) => <DisplayComponent
        className={prop.className}
        key={key}
        label={prop.label}
        value={componentState[prop.name]}
    />;

    render() {
        const {children, className, currentState, modelName, updateState} = this.props;
        const {
            displayedProperties,
            extraBindingSections,
            navigationDescription,
            navigationTitle,
            type
        } = currentState[modelName];

        const componentModel = currentState[modelName];

        const updateProps = (value) => {
            updateState(modelName, {...currentState[modelName], ...value});
        };

        const displayBindings = () => <div className="testBindings">
            {RU.mapIndexed(this.displayItem(componentModel), displayedProperties)}
            {createExtraBindingSections(extraBindingSections, currentState[modelName], R.T, updateProps)}
        </div>;

        const childrenWithProps = React.Children.map(children,
            (child) => React.cloneElement(child, {currentState, modelName, updateState})
        );

        return <div>
            <ElementNavTitle
                description={navigationDescription}
                modelName={modelName}
                title={navigationTitle}
                type={type}
            />
            <div className={className}>
                <div className="testComponent">
                    {childrenWithProps}
                </div>
                {displayBindings()}
            </div>
        </div>;
    }
}
