import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {modelActionsForDropdownOnlyButton} from '../buttons-page/button-initial-state';
import {XlReactWidgetString} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactButtonWithDropdown} from '../../xl-react-buttons/xl-react-button-with-dropdown/xl-react-button-with-dropdown'; // eslint-disable-line max-len
import KitchenTestComponent from './kitchen-test-component';

export class WidgetStringWithButtonAddonTestComponent extends PureComponent {
    static propTypes = {
        className: PropTypes.string.isRequired,
        currentState: PropTypes.object.isRequired,
        label: PropTypes.string.isRequired,
        modelName: PropTypes.string.isRequired,
        updateState: PropTypes.func.isRequired
    };

    handleModelChange = (value) => {
        const {currentState, modelName} = this.props;
        this.changeModel({
            button: R.pathOr('', [modelName, 'model', 'button'], currentState),
            filter: value
        });
    };

    onButtonClicked = (modelName, value) => {
        this.changeModel({button: value});
    };

    changeModel = (model) => {
        const {currentState, modelName, updateState} = this.props;
        updateState(modelName, R.mergeDeepRight(currentState[modelName], {model}));
    };

    render() {
        const {className, label, modelName, currentState, updateState} = this.props;
        const actions = modelActionsForDropdownOnlyButton(modelName, this.onButtonClicked);

        return <KitchenTestComponent
            className={className}
            currentState={currentState}
            key={modelName}
            label={label}
            modelName={modelName}
            options={{copyModel: false}}
            updateState={updateState}>
            <XlReactWidgetString
                addons={
                    <XlReactButtonWithDropdown
                        actions={actions}
                        disabled={currentState[modelName].disabled}
                        type="secondary"
                    />
                }
                model={currentState[modelName].model.filter}
                onModelChange={this.handleModelChange}
            />
        </KitchenTestComponent>;
    }
}
