/* eslint-disable sort-keys, object-curly-newline, no-multi-str, max-lines */
import R from 'ramda';
import moment from 'moment';
import KeyCodes from 'keycode-js';
import variables from '../../../app/styles/variables';
import {localeMessages} from '../../services/locale-service';
import {
    xlReactWidgetTableFilterStyle,
    xlReactWidgetTableFilterTypes
} from '../../xl-react-widget/xl-react-widget-redux-table/xl-react-widget-table-filter-type';
import {generateTree} from '../playground/playground-utils';
import RU from '../../utils/ramda-utils';
import hostname from '../../../gulp/utils/hostname';
import {
    widgetStringValues,
    widgetStringViews
} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string-enum';
import {tagsAutocompleteSkins} from '../../xl-react-widget/xl-react-widget-tags/xl-react-widget-tags-skins';
import {confirmationDialogSkins}
    from '../../xl-react-widget/xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog-skins';
import {genericDateFormat} from '../../utils/date-utils';
import {SIDES} from '../../xl-react-widget/xl-react-widget-split-panel/xl-react-widget-collapsible-vertical-split-pane/xl-react-widget-collapsible-vertical-split-pane-constants'; // eslint-disable-line max-len
import jsonSample from './sample.json';
import yamlSample from './sample.yaml';

const type = 'widget';

const widgetDate1MinimumDate = moment({year: 2016, month: 10, day: 2});
const widgetDate1MaximumDate = moment({year: 2016, month: 10, day: 20});
const widgetDate1CurrentDate = moment({year: 2016, month: 10, day: 9});
const widgetTime1CurrentTime = moment({
    year: 2016, month: 10, day: 9, hours: 15, minutes: 10, seconds: 0, milliseconds: 0
});

export const widgetReduxTableData = [{
    firstname: 'John A. Smith',
    address: '1236 Some Street',
    city: 'San Francisco',
    state: 'CA'
}, {
    firstname: 'Joan B. Jones',
    address: '3271 Another Ave',
    city: 'New York',
    state: 'NY'
}, {
    firstname: 'Bob C. Uncle',
    address: '9996 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'John D. Smith',
    address: '1623 Some Street',
    city: 'San Francisco',
    state: 'CA'
}, {
    firstname: 'Bob F. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob G. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob H. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob I. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob J. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'John Doe',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}];

export const autocompleteElements = R.map((ind) => ({
    id: `Application${ind}`,
    type: ind
}), R.range(1, 7));

export const widgetInitialState = {
    widgetAlert1: {
        alertType: 'success',
        navigationDescription: 'xl-react-widget-alert-message',
        navigationTitle: 'Alerts widget',
        type
    },
    widgetCategory: {
        open: true,
        navigationDescription: 'xl-react-widget-category',
        navigationTitle: 'Collapsible panel',
        type
    },
    widgetRadioGroup1: {
        $hiddenDisplayedComponents: ['Valid', 'Required', 'Show messages', 'Display Mode', 'API'],
        model: [{
            color: 'blue',
            label: 'Case 1',
            value: 'Case 1'
        }, {
            checked: true,
            color: 'yellow',
            label: 'Case 2',
            value: 'Case 2'
        }, {
            color: 'red',
            label: 'Case 3',
            value: 'Case 3'
        }],
        navigationDescription: 'xl-react-widget-radio',
        navigationTitle: 'Radio Widget',
        type
    },
    widgetString1: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            default: '',
            description: 'This is my string field.',
            label: 'String me',
            placeholder: 'String placeholder',
            type: 'text'
        },
        model: '',
        navigationDescription: 'xl-react-widget-string',
        navigationTitle: 'String Widget',
        pristine: true,
        valid: true,
        extraBindingSections: [
            {
                clazzName: 'change-type',
                label: 'Input Type',
                name: 'metadata.type',
                type: 'enum',
                enum: ['password', 'text'],
                value: 'text'
            },
            {
                clazzName: 'change-size',
                label: 'Size',
                model: '',
                name: 'metadata.size',
                type: 'select',
                values: R.values(widgetStringValues)
            },
            {
                clazzName: 'change-view',
                label: 'View',
                model: '',
                name: 'metadata.view',
                type: 'select',
                values: R.values(widgetStringViews)
            },
            {
                clazzName: 'default-value',
                label: 'Default value',
                name: 'metadata.default',
                onChangeCallApi: (api, value) => {
                    RU.invoke('applyDefaultModel', [value], api);
                },
                type: 'input',
                value: ''
            }
        ],
        type
    },
    widgetString2: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            default: '',
            description: 'This is my string field.',
            label: 'String with addon',
            placeholder: 'String placeholder',
            type: 'text'
        },
        model: '',
        navigationDescription: 'xl-react-widget-string-with-addon',
        navigationTitle: 'String widget with addon',
        pristine: true,
        valid: true,
        extraBindingSections: [
            {
                clazzName: 'change-type',
                label: 'Input Type',
                name: 'metadata.type',
                type: 'enum',
                enum: ['password', 'text'],
                value: 'text'
            },
            {
                clazzName: 'change-size',
                label: 'Size',
                model: '',
                name: 'metadata.size',
                type: 'select',
                values: R.values(widgetStringValues)
            },
            {
                clazzName: 'default-value',
                label: 'Default value',
                name: 'metadata.default',
                onChangeCallApi: (api, value) => {
                    RU.invoke('applyDefaultModel', [value], api);
                },
                type: 'input',
                value: ''
            }
        ],
        type
    },
    widgetStringWithButtonAddon: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            default: '',
            description: 'This is my string field.',
            label: 'String with button addon',
            placeholder: 'String placeholder'
        },
        model: {
            button: '',
            filter: ''
        },
        navigationDescription: 'xl-react-widget-string-with-button-addon',
        navigationTitle: 'String widget with button addon',
        pristine: true,
        valid: true,
        type
    },
    widgetPatchSample: {
        displayedProperties: [{
            className: 'selected-lines',
            label: 'Selected lines',
            name: 'selectedLines'
        }],
        filePath: 'Environments/Patch-1',
        headerLabel: 'Sample',
        model: yamlSample,
        navigationDescription: 'xl-react-widget-patch-sample',
        navigationTitle: 'Patch sample',
        selectedLines: [],
        type
    },
    widgetPatchEditor: {
        displayedProperties: [{
            className: 'pristine',
            label: 'Pristine',
            name: 'pristine'
        }, {
            className: 'selected-lines',
            label: 'Selected lines',
            name: 'selectedLines'
        }, {
            className: 'selected-action',
            label: 'Selected action',
            name: 'selectedAction'
        }, {
            className: 'file-type',
            label: 'File type',
            name: 'fileType'
        }, {
            className: 'last-pushed-patch',
            label: 'Last pushed patch',
            name: 'lastPushedPatch'
        }],
        editableSection: '',
        extraBindingSections: [
            {
                clazzName: 'file-type',
                label: 'File type',
                name: 'fileType',
                type: 'enum',
                enum: ['yaml', 'json'],
                value: 'yaml'
            }
        ],
        filePath: 'Environments/Patch-1',
        fileType: 'yaml',
        initialYamlModel: yamlSample,
        initialJsonModel: jsonSample,
        lastPushedPatch: '',
        model: yamlSample,
        navigationDescription: 'xl-react-widget-patch-editor',
        navigationTitle: 'Patch editor',
        pristine: true,
        selectedAction: 'none',
        selectedLines: [],
        selectedPath: '',
        type
    },
    widgetProgress1: {
        $hiddenDisplayedComponents: ['Required', 'Show messages'],
        disabled: false,
        displayMode: 'edit',
        metadata: {
            description: 'This is my string field.',
            label: 'I\'m required string',
            placeholder: 'String placeholder',
            required: true
        },
        model: 150,
        minValue: 0,
        maxValue: 200,
        navigationDescription: 'xl-react-widget-progressbar',
        navigationTitle: 'Progress bar',
        extraBindingSections: [
            {
                type: 'integer',
                label: 'Current progress'
            },
            {
                type: 'boolean',
                label: 'Stripped',
                name: 'stripped',
                value: false
            },
            {
                type: 'enum',
                label: 'Type',
                name: 'currentStatus',
                value: 'progress',
                enum: [
                    'progress',
                    'success',
                    'error'
                ]
            }
        ],
        type
    },
    widgetFilter1: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            label: 'I\'m a filter',
            placeholder: 'I\'m a placeholder'
        },
        model: '',
        navigationDescription: 'xl-react-widget-filter',
        navigationTitle: 'Filter widget',
        type
    },
    widgetTags1: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            description: 'Click me.',
            default: R.map(
                (value) => ({label: value, value}),
                ['test']
            ),
            label: 'insert a tag',
            name: 'tags.test',
            required: true
        },
        model: [],
        navigationDescription: 'xl-react-widget-tags',
        navigationTitle: 'Tags Widget',
        type
    },
    widgetTags2: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            description: 'Click me.',
            default: [],
            label: 'insert a tag',
            name: 'tags.test',
            required: true,
            requiredText: 'Custom required text overriding default required text!'
        },
        model: [],
        navigationDescription: 'xl-react-widget-tags',
        navigationTitle: 'Tags Widget (Custom default error)',
        type
    },
    widgetTags3: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            label: 'Tags with autocomplete',
            name: 'widgetTags3',
            description: 'These are autocomplete tags.',
            placeholder: 'placeholder'
        },
        model: [{
            color: variables['xl-green'],
            label: 'Application 0',
            value: 'Applications/TestApp/Application 0'
        }],
        navigationDescription: 'xl-react-widget-tags',
        navigationTitle: 'Tags Widget (Autocomplete input)',
        extraBindingSections: [
            {
                enum: R.values(tagsAutocompleteSkins),
                label: 'Skin',
                name: 'skin',
                type: 'enum',
                value: tagsAutocompleteSkins.CHECKBOX
            },
            {
                type: 'boolean',
                label: 'Only unique suggestion',
                name: 'allowAddOnlyUniqueSuggestion',
                value: false
            }
        ],
        type
    },
    widgetDate1: {
        disabled: false,
        displayMode: 'edit',
        minimumDate: widgetDate1MinimumDate,
        maximumDate: widgetDate1MaximumDate,
        model: widgetDate1CurrentDate,
        navigationDescription: 'xl-react-widget-date',
        navigationTitle: 'Date picker Widget',
        metadata: {
            description: 'Date picker description',
            name: 'my date',
            label: 'Choose a date',
            format: genericDateFormat
        },
        extraBindingSections: [
            {
                className: 'date',
                label: 'Minimum date',
                name: 'minDate',
                type: 'date',
                value: widgetDate1MinimumDate
            },
            {
                className: 'date',
                label: 'Maximum date',
                name: 'maxDate',
                type: 'date',
                value: widgetDate1MaximumDate
            }
        ],
        type
    },
    widgetTime1: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            label: 'Choose time',
            description: 'Widget time description'
        },
        model: widgetTime1CurrentTime,
        navigationDescription: 'xl-react-widget-time',
        navigationTitle: 'Time picker Widget',
        type
    },
    widgetAutocomplete1: {
        disabled: false,
        displayMode: 'edit',
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Strict validation',
                name: 'metadata.strictToOptions',
                value: false
            }
        ],
        metadata: {
            label: 'Autocomplete',
            name: 'application.autocomplete',
            description: 'This is my autocomplete field.',
            placeholder: 'autocomplete placeholder',
            required: true,
            requiredText: 'Custom required error message overrides!'
        },
        navigationDescription: 'xl-react-widget-autocomplete',
        navigationTitle: 'Autocomplete Widget',
        type
    },
    widgetAutocomplete2: {
        disabled: false,
        displayMode: 'edit',
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Strict validation',
                name: 'metadata.strictToOptions',
                value: false
            }
        ],
        metadata: {
            default: 'Application2',
            label: 'Autocomplete',
            name: 'application.autocomplete',
            description: 'This is my autocomplete field.',
            placeholder: 'autocomplete placeholder',
            required: true,
            requiredText: 'Custom required error message overrides!'
        },
        navigationDescription: 'xl-react-widget-autocomplete',
        navigationTitle: 'Autocomplete Widget (Default value)',
        type
    },
    widgetAutocomplete3: {
        disabled: false,
        displayMode: 'edit',
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Strict validation',
                name: 'metadata.strictToOptions',
                value: false
            }
        ],
        metadata: {
            default: {id: 'Application2', type: 2},
            label: 'Autocomplete',
            name: 'application.autocomplete',
            description: 'This is my autocomplete field.',
            placeholder: 'autocomplete placeholder',
            required: true,
            requiredText: 'Custom required error message overrides!'
        },
        navigationDescription: 'xl-react-widget-autocomplete',
        navigationTitle: 'Autocomplete Widget Model Objects (Default value)',
        type
    },
    widgetAutocomplete4: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            label: 'Autocomplete',
            name: 'application.autocomplete',
            description: 'This is my autocomplete field.',
            placeholder: 'autocomplete placeholder',
            required: true,
            requiredText: 'Custom required error message overrides!'
        },
        navigationDescription: 'xl-react-widget-autocomplete-with-addons',
        navigationTitle: 'Autocomplete Widget with addons',
        type
    },
    widgetAutocomplete5: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            label: 'Autocomplete',
            name: 'application.autocomplete.delayed',
            description: 'This is my autocomplete field.',
            placeholder: 'autocomplete placeholder',
            required: true,
            requiredText: 'Custom required error message overrides!'
        },
        navigationDescription: 'xl-react-widget-autocomplete-with-delay',
        navigationTitle: 'Autocomplete Widget with delay (spinner)',
        type
    },
    widgetFileUpload1: {
        disabled: false,
        displayMode: 'edit',
        metadata: {
            auto: false,
            browseButtonType: 'secondary',
            label: 'Upload me',
            description: 'File Upload description',
            uploadUrl: `http://${hostname}:3002/upload`
        },
        navigationDescription: 'xl-react-widget-file-upload',
        navigationTitle: 'File upload Widget',
        type
    },
    widgetModalDialog1: {
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Animate',
                name: 'animation',
                value: false
            }
        ],
        navigationDescription: 'xl-react-widget-modal-dialog',
        navigationTitle: 'Modal Dialog Widget',
        title: 'Modal dialog',
        text: 'Lorem Ipsum is simply dummy text of the printing and typesetting industry. \
        Lorem Ipsum has been the industry\'s standard dummy text ever since the 1500s, when \
        an unknown printer took a galley of type and scrambled it to make a type specimen book. \
        It has survived not only five centuries, but also the leap into electronic typesetting, \
        remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset \
        sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like \
        Aldus PageMaker including versions of Lorem Ipsum.',
        $mapping: {
            showPropertyName: 'showModal',
            handleHideFunctionNames: [
                'handleClose'
            ]
        },
        type
    },
    widgetModalDialog2: {
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Animate',
                name: 'animation',
                value: false
            }
        ],
        expandable: true,
        navigationDescription: 'xl-react-widget-modal-dialog',
        navigationTitle: 'Expandable Modal Dialog Widget',
        title: 'Expandable modal dialog',
        text: 'Lorem Ipsum is simply dummy text of the printing and typesetting industry. \
        Lorem Ipsum has been the industry\'s standard dummy text ever since the 1500s, when \
        an unknown printer took a galley of type and scrambled it to make a type specimen book. \
        It has survived not only five centuries, but also the leap into electronic typesetting, \
        remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset \
        sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like \
        Aldus PageMaker including versions of Lorem Ipsum.',
        $mapping: {
            showPropertyName: 'showModal',
            handleHideFunctionNames: [
                'handleClose'
            ]
        },
        type
    },
    widgetConfirmationDialog1: {
        buttons: [
            {
                action: 'confirm',
                className: 'xl-primary',
                label: 'Yes',
                keyCodes: [KeyCodes.KEY_RETURN]
            },
            {
                action: 'reject',
                className: 'xl-secondary',
                label: 'No',
                keyCodes: [KeyCodes.KEY_ESCAPE]
            },
            {
                action: 'noop',
                className: '',
                disabled: true,
                label: 'Disabled button',
                keyCodes: [KeyCodes.KEY_0]
            }
        ],
        extraBindingSections: [
            {
                type: 'boolean',
                label: 'Animate',
                name: 'animation',
                value: false
            },
            {
                enum: R.values(confirmationDialogSkins),
                label: 'Skin',
                name: 'skin',
                type: 'enum',
                value: confirmationDialogSkins.DEFAULT
            }
        ],
        navigationDescription: 'xl-react-widget-confirmation-dialog',
        navigationTitle: 'Confirmation Dialog Widget',
        title: 'Confirmation dialog with a long title name which doesn\'t fit the default size of modal window',
        text: 'Are you sure you want to leave? Your changes will be lost.',
        $mapping: {
            showPropertyName: 'show',
            handleHideFunctionNames: ['onButtonClick', 'onCheckboxClick']
        },
        type
    },
    widgetTree1: {
        autoOpen: false,
        className: 'xl-react-widget-tree',
        model: [
            {
                id: 'node--1',
                children: generateTree(2, 3),
                name: 'Pre-Loaded'
            },
            {
                id: 'node--2',
                name: 'Empty-Node'
            }
        ],
        navigationDescription: 'xl-react-widget-tree',
        navigationTitle: 'Tree Widget',
        selectedNodes: [],
        selectionMode: 'multi',
        selectable: true,
        type
    },
    widgetTree2: {
        autoOpen: false,
        className: 'xl-react-widget-tree-lazy',
        model: [
            {
                id: 'node--1',
                loadOnDemand: true,
                name: 'Lazy-Loaded'
            }
        ],
        navigationDescription: 'xl-react-widget-tree',
        navigationTitle: 'Lazy Tree Widget',
        selectedNodes: [],
        selectionMode: 'multi',
        selectable: true,
        type
    },
    widgetBottomSplitPane: {
        defaultPaneHeight: '80%',
        minSizeBottom: 27,
        minSizeTop: 35,
        navigationDescription: 'xl-react-widget-bottom-split-pane',
        navigationTitle: 'Horizontal split pane Widget',
        showBindings: false,
        type
    },
    widgetVerticalSplitPane: {
        isCollapsed: false,
        minPrimarySideWidth: 100,
        minSecondarySideWidth: 100,
        navigationDescription: 'xl-react-widget-collapsible-vertical-split-pane',
        navigationTitle: 'Vertical split pane Widget',
        primary: SIDES.primary,
        primarySideWidth: 100,
        shouldAnimate: true,
        showBindings: false,
        sideToCollapse: SIDES.primary,
        type
    },
    widgetTable: {
        data: [{
            firstname: 'John A. Smith',
            address: '1236 Some Street',
            city: 'San Francisco',
            state: 'CA'
        }, {
            firstname: 'Joan B. Jones',
            address: '3271 Another Ave',
            city: 'New York',
            state: 'NY'
        }, {
            firstname: 'Bob C. Uncle',
            address: '9996 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'John D. Smith',
            address: '1623 Some Street',
            city: 'San Francisco',
            state: 'CA'
        }, {
            firstname: 'Bob F. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob G. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob H. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob I. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob J. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'John Doe',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }],
        fieldFormatters: {
            completionDate: (arg) => arg,
            startDate: (arg) => arg,
            status: (arg) => arg,
            type: (arg) => arg
        },
        filters: {
            firstname: '',
            address: '',
            city: '',
            state: ''
        },
        keyField: 'firstname',
        layout: RU.mapIndexed(([key, title], id) => ({
            dataKey: key,
            sortKey: key,
            title: title,
            id
        }), [['firstname', 'Firstname'], ['address', 'Address'], ['city', 'City'], ['state', 'State']]),
        loading: false,
        loadingMessage: localeMessages.tableLoadingMessage,
        navigationDescription: 'xl-react-widget-table',
        navigationTitle: 'Table Widget',
        paging: {
            page: 1,
            pageSizes: [{
                text: '2',
                value: 2
            }, {
                text: '5',
                value: 5
            }, {
                text: '10',
                value: 10
            }],
            pageStartIndex: 1,
            resultsPerPage: 2,
            prePageText: '<',
            nextPageText: '>',
            paginationPosition: 'bottom'
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'startDate', order: 'desc'},
        total: 10,
        type
    },
    widgetReduxTable: {
        columnsFilter: {
            firstname: '',
            address: '',
            city: '',
            state: ''
        },
        data: R.slice(0, 2, widgetReduxTableData),
        fieldFormatters: {},
        filters: [
            {
                className: '',
                fieldPath: 'firstname-filter-key',
                label: 'First name',
                key: 'firstname',
                model: '',
                placeholder: 'Filter by first name',
                position: {
                    column: 0,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'address',
                label: 'Address',
                key: 'address',
                model: '',
                placeholder: 'Filter by address',
                position: {
                    column: 0,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: () => 'city',
                label: 'City',
                key: 'city',
                model: '',
                placeholder: 'Filter by city',
                position: {
                    column: 1,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'state',
                label: 'State',
                key: 'state',
                model: '',
                placeholder: 'Filter by state',
                position: {
                    column: 1,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            }
        ],
        filterType: 'above',
        keyField: 'firstname',
        layout: RU.mapIndexed(
            ([key, title, sortKey], id) => ({
                dataKey: key,
                sortKey,
                title: title,
                id
            }),
            [
                ['firstname', 'Firstname', 'firstname'],
                ['address', 'Address', 'address-sort-key'],
                ['city', 'City', 'city'],
                ['state', 'State', 'state']
            ]
        ),
        loading: false,
        navigationDescription: 'xl-react-widget-redux-table',
        navigationTitle: 'Redux Table Widget',
        paging: {
            page: 1,
            pageSizes: [
                {
                    text: '2',
                    value: 2
                },
                {
                    text: '5',
                    value: 5
                },
                {
                    text: '10',
                    value: 10
                }
            ],
            resultsPerPage: 2
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'firstname', order: 'desc'},
        tableId: 'redux-table-1',
        total: 10,
        type
    },
    widgetReduxTable2: {
        data: R.slice(0, 2, widgetReduxTableData),
        fieldFormatters: {},
        filters: [
            {
                className: '',
                fieldPath: 'firstname-filter-key',
                label: 'First name',
                key: 'firstname',
                model: '',
                placeholder: 'Filter by first name',
                position: {
                    column: 0,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'address',
                label: 'Address',
                key: 'address',
                model: '',
                placeholder: 'Filter by address',
                position: {
                    column: 0,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: () => 'city',
                label: 'City',
                key: 'city',
                model: '',
                placeholder: 'Filter by city',
                position: {
                    column: 1,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.AUTOCOMPLETE
            },
            {
                className: '',
                fieldPath: 'state',
                label: 'State',
                key: 'state',
                model: [],
                placeholder: 'Filter by state',
                position: {
                    column: 1,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.TAGS
            }
        ],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        filterProps: {
            state: {
                handlers: {
                    autocomplete: {
                        addCandidates: () => Promise.resolve([
                            {value: 'CA'},
                            {value: 'NY'}
                        ])
                    }
                },
                objLabel: 'value',
                skin: tagsAutocompleteSkins.CHECKBOX
            },
            city: {
                handlers: {
                    addCandidates: () => Promise.resolve([
                        {id: 'Los Angeles'},
                        {id: 'New York'},
                        {id: 'San Francisco'}
                    ])
                }
            }
        },
        keyField: 'firstname',
        layout: RU.mapIndexed(
            ([key, title, sortKey], id) => ({
                dataKey: key,
                sortKey,
                title: title,
                id
            }),
            [
                ['firstname', 'Firstname', 'firstname'],
                ['address', 'Address', 'address-sort-key'],
                ['city', 'City', 'city'],
                ['state', 'State', 'state']
            ]
        ),
        loading: false,
        navigationDescription: 'xl-react-widget-redux-table-with-autocomplete-filter',
        navigationTitle: 'Redux Table Widget with autocomplete filters',
        paging: {
            page: 1,
            pageSizes: [
                {
                    text: '2',
                    value: 2
                },
                {
                    text: '5',
                    value: 5
                },
                {
                    text: '10',
                    value: 10
                }
            ],
            resultsPerPage: 2
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'firstname', order: 'desc'},
        tableId: 'redux-table-2',
        total: 10,
        type
    },
    widgetReduxTable3: {
        data: [{
            dateOfBirth: moment({year: 2016, month: 10, day: 2})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 3})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 4})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 5})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 6})
        }],
        fieldFormatters: {dateOfBirth: (thisMoment) => thisMoment.toString()},
        filters: [{
            className: '',
            fieldPath: 'afterDateOfBirth',
            label: 'Date Of Birth after',
            key: 'afterDateOfBirth',
            model: moment({year: 2016, month: 10, day: 3}),
            placeholder: 'Date Of Birth after',
            position: {
                column: 0,
                row: 0
            },
            type: xlReactWidgetTableFilterTypes.DATEPICKER
        }, {
            className: '',
            fieldPath: 'beforeDateOfBirth',
            label: 'Date Of Birth before',
            key: 'beforeDateOfBirth',
            model: moment({year: 2016, month: 10, day: 4}),
            placeholder: 'Date Of Birth before',
            position: {
                column: 1,
                row: 0
            },
            type: xlReactWidgetTableFilterTypes.DATEPICKER
        }],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        keyField: 'dateOfBirth',
        layout: [{
            dataKey: 'dateOfBirth',
            dataSort: false,
            sortKey: 'dateOfBirth',
            title: 'Date Of Birth'
        }],
        loading: false,
        navigationDescription: 'xl-react-widget-redux-table-with-date-picker',
        navigationTitle: 'Redux Table Widget with Date Picker',
        paging: {
            page: 1,
            pageSizes: [{
                text: '5',
                value: 5
            }],
            pageStartIndex: 1,
            resultsPerPage: 5
        },
        sorting: {name: 'dateOfBirth', order: 'asc'},
        tableId: 'redux-table-3',
        total: 5,
        type
    },
    widgetReduxTable4: {
        data: R.range(1, 101).map(R.objOf('value')),
        filters: [{
            className: '',
            fieldPath: 'value',
            label: 'Condition',
            key: 'value',
            model: 'all',
            position: {
                column: 0,
                row: 0
            },
            metadata: [
                {label: 'All', model: 'all'},
                {label: 'Value < 50', model: 'lt50'},
                {label: 'Value > 50', model: 'gt50'},
                {label: 'Value is greater than 50', model: 'gt50'}
            ],
            objLabel: 'label',
            objValue: 'model',
            type: xlReactWidgetTableFilterTypes.TOGGLE_BUTTONS
        }],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        keyField: 'value',
        layout: [{
            dataKey: 'value',
            dataSort: false,
            sortKey: 'value',
            title: 'Value'
        }],
        loading: false,
        navigationDescription: 'xl-react-widget-redux-table-with-toggle-button-filter',
        navigationTitle: 'Redux Table Widget with Toggle Button filter',
        paging: {
            page: 1,
            pageSizes: [{
                text: '5',
                value: 5
            }],
            pageStartIndex: 1,
            resultsPerPage: 5
        },
        sorting: {name: 'value', order: 'asc'},
        tableId: 'redux-table-4',
        total: 100,
        type
    },
    widgetTreeSelect: {
        data: [
            {
                id: 'FOLDER_1',
                title: 'Folder 1',
                enabled: true
            },
            {
                children: [{
                    children: [{
                        children: [{
                            id: 'FOLDER_2111',
                            title: 'Folder 2111',
                            enabled: false
                        }, {
                            id: 'FOLDER_2112',
                            title: 'Folder 2112',
                            enabled: false
                        }],
                        id: 'FOLDER_211',
                        title: 'Folder 211',
                        enabled: false
                    }, {
                        id: 'FOLDER_212',
                        title: 'Folder 212',
                        enabled: false
                    }, {
                        id: 'FOLDER_213',
                        title: 'Folder 213',
                        enabled: false
                    }],
                    id: 'FOLDER_21',
                    title: 'Folder 21',
                    enabled: false
                }, {
                    children: [{
                        id: 'FOLDER_222',
                        title: 'Folder 222',
                        enabled: false
                    }],
                    id: 'FOLDER_22',
                    title: 'Folder 22',
                    enabled: true
                }],
                id: 'FOLDER_2',
                title: 'Folder 2',
                enabled: true
            },
            {
                id: 'FOLDER_3',
                title: 'Folder 3',
                enabled: false
            },
            {
                id: 'FOLDER_4',
                title: 'Folder 4',
                enabled: false
            },
            {
                id: 'FOLDER_5',
                title: 'Folder 5',
                enabled: false
            },
            {
                children: [{
                    children: [{
                        children: [{
                            id: 'FOLDER_6111',
                            title: 'Folder 6111',
                            enabled: false
                        }, {
                            id: 'FOLDER_6112',
                            title: 'Folder 6112',
                            enabled: false
                        }],
                        id: 'FOLDER_611',
                        title: 'Folder 611',
                        enabled: false
                    }],
                    id: 'FOLDER_61',
                    title: 'Folder 61',
                    enabled: false
                }],
                id: 'FOLDER_6',
                title: 'Folder 6',
                enabled: false
            }
        ],
        model: {
            id: 'FOLDER_6111',
            title: 'Folder 6111',
            enabled: true
        },
        navigationDescription: 'xl-react-widget-tree-select',
        navigationTitle: 'Folder Tree Select Widget',
        type
    },
    widgetTreeSelectItem: {
        isFocused: true,
        navigationTitle: 'Tree Select Item',
        node: {
            depth: 0,
            id: 'FOLDER_2',
            isEnabled: true,
            isExpanded: true,
            isParent: true,
            isSelected: true,
            title: 'FOLDER 2'
        },
        type
    },
    widgetSpinner: {
        navigationDescription: 'xl-react-widget-spinner',
        navigationTitle: 'Spinner widget',
        overlay: true,
        message: localeMessages.spinnerMessage,
        show: true,
        type
    },
    widgetSwitch: {
        displayMode: 'edit',
        navigationDescription: 'xl-react-widget-switch',
        navigationTitle: 'Widget switch',
        metadata: {label: 'Switch button'},
        model: true,
        pristine: true,
        valid: true,
        type
    },
    widgetWithPopover: {
        navigationDescription: 'xl-react-widget-with-popover',
        navigationTitle: 'Widget with Popover',
        popoverContent: 'Widget with popover content',
        type
    },
    widgetMonacoDiffViewer: {
        navigationDescription: 'xl-react-widget-monaco-diff-viewer',
        navigationTitle: 'Monaco diff viewer',
        value: 'Value',
        original: 'Original',
        type
    },
    widgetMonacoEditor: {
        navigationDescription: 'xl-react-widget-monaco-editor',
        navigationTitle: 'Monaco editor',
        value: '',
        displayedProperties: [{
            className: 'value',
            label: 'Value',
            name: 'value'
        }],
        type,
        extraBindingSections: [
            {
                clazzName: 'readonly',
                label: 'Read only',
                name: 'options.readonly',
                type: 'boolean',
                value: true
            }
        ],
        options: {readonly: true}
    }
};
