import PropTypes from 'prop-types';
import React, {Component} from 'react';
import {isComponentDisabled, resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipBooleanComponent} from './xl-react-dip-boolean-component';

export class XlReactDipBoolean extends Component {
    static propTypes = {
        model: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {};
    }

    componentWillMount() {
        computeInitialState(this, this.props, getValidators(this.props), this.props.model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
    }

    onToggleChange = () => {
        computeNextState(this, this.props, getValidators(this.props), !this.state.model);
    };

    api = {resetPristinity: (config) => resetPristinity(this, this.props, config)};

    render() {
        const {metadata: {description, label, name, required}, onValidityChange} = this.props;
        const {model, message, valid} = this.state;

        const disabled = isComponentDisabled(this.props);

        return <XlReactDipBooleanComponent {...({
            description,
            disabled,
            handleModelChange: this.onToggleChange,
            label,
            message,
            model,
            name,
            onValidityChange,
            required,
            valid
        })}
        />;
    }
}
