import R from 'ramda';
import PropTypes from 'prop-types';
import React, {Component} from 'react';
import moment from 'moment';
import RU from '../../utils/ramda-utils';
import {resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipDatetimeComponent} from './xl-react-dip-datetime-component';

export class XlReactDipDatetime extends Component {
    static propTypes = {
        model: PropTypes.object,
        ...XlReactDipContainerProps
    };
    static defaultProps = {
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {};
        this.apiData = {
            date: undefined,
            time: undefined
        };

        this.validityData = {
            date: true,
            time: true
        };

        this.pristinityData = {
            date: true,
            time: true
        };
    }

    componentWillMount() {
        computeInitialState(this, this.props, getValidators(this.props), this.props.model, false);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps), false);
    }

    handleDateChange = (newDate) => {
        const newDateTime = moment(this.state.model);
        if (newDate && newDateTime.isValid()) {
            newDateTime.date(newDate.getDate());
            newDateTime.month(newDate.getMonth());
            newDateTime.year(newDate.getFullYear());
            this.setNewDateTime(newDateTime.toDate());
        } else {
            this.setNewDateTime(newDate);
        }
    };

    handleTimeChange = (newTime) => {
        const newDateTime = moment(this.state.model);
        if (moment(newTime)
            .isValid()) {
            newDateTime.milliseconds(newTime.getMilliseconds());
            newDateTime.seconds(newTime.getSeconds());
            newDateTime.minutes(newTime.getMinutes());
            newDateTime.hours(newTime.getHours());
            this.setNewDateTime(newDateTime.toDate());
        }
    };

    onPristinityChanged = (component, pristine) => {
        const oldPristinity = this.pristinityData;
        const newPristinity = {
            ...this.pristinityData,
            [component]: pristine
        };

        const wasPristineBefore = RU.areAllTrue(oldPristinity);
        const isNowPristine = RU.areAllTrue(newPristinity);

        this.setState({pristine: isNowPristine});
        this.pristinityData = newPristinity;

        if (wasPristineBefore !== isNowPristine) {
            RU.invoke('onPristinityChange', [isNowPristine], this.props);
        }
    };

    onValidityChanged = (component, valid) => {
        const oldValidity = this.validityData;
        const newValidity = {
            ...this.validityData,
            [component]: valid
        };

        const wasValidBefore = RU.areAllTrue(oldValidity);
        const isNowValid = RU.areAllTrue(newValidity);

        this.setState({valid: isNowValid});
        this.validityData = newValidity;

        if (wasValidBefore !== isNowValid) {
            RU.invoke('onValidityChange', [isNowValid], this.props);
        }
    };

    api = {
        focus: () => this.apiData.date.focus(),
        resetPristinity: (config) => {
            RU.invoke(['apiData', 'date', 'resetPristinity'], [config], this);
            RU.invoke(['apiData', 'time', 'resetPristinity'], [config], this);

            resetPristinity(this, this.props, config);
        }
    };

    messageHandler = (dateErrorMessage) => {
        this.setState({dateErrorMessage});
    };

    register = (component, componentApi) => {
        this.apiData = {
            ...this.apiData,
            [component]: componentApi
        };
        RU.invoke(['register'], [this.api], this.props);
    };

    setNewDateTime = (newDateTime) => {
        RU.invoke('onModelChange', [newDateTime], this.props);
        this.setState({model: newDateTime});
    };

    render() {
        const {dateErrorMessage, message, model} = this.state;
        const {
            displayMode, disabled,
            metadata: {description, label, required, time}, onMessageChange
        } = this.props;

        const messageObj = dateErrorMessage || message;
        return <XlReactDipDatetimeComponent {...({
            description,
            disabled,
            displayMode,
            handleDateChange: this.handleDateChange,
            handleTimeChange: this.handleTimeChange,
            invalid: !R.isNil(dateErrorMessage),
            label,
            message: {...messageObj, show: true},
            messageHandler: this.messageHandler,
            model,
            onMessageChange,
            onPristinityChange: this.onPristinityChanged,
            onValidityChange: this.onValidityChanged,
            register: this.register,
            required,
            time
        })}
        />;
    }
}
