import Form from '../../dsl/component/form/form';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';

describe('XL Dip Dynamic Form With Lookup', () => {
    beforeAll(() => {
        Navigation.openHomePage();
        Navigation.openFormElement('dynamicWithLookup');
    });

    it('should have right initial state', () => {
        TestBindings.expectObjectModelToEqual({
            encryptedMap: {key: 'secret'},
            'external-properties': {
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                }
            },
            largePassword: '',
            password: ''
        });

        const providerId = 'Configurations/Vault/LookupValueProvider';
        const label = 'Password field with lookup value';

        Form.expectLookupFieldHasValue('Password field', 'input', '');
        Form.expectLookupFieldHasValue('Large password field', 'textarea', '');
        Form.expectToggledInputLookupHasValues(label, providerId, 'SECRET');
        Form.expectLookupHasWarning(label, 'Invalid vault provider value');
        Form.expectFieldsArePristine(['password', 'largePassword', 'vault', 'encryptedMap']);
    });

    it('should change field pristine state on toggle and update lookup model', () => {
        Form.toggleLookup('Password field');
        Form.expectedFieldsPristinity({
            password: false,
            largePassword: true,
            vault: true,
            encryptedMap: true
        });

        Form.expectModelToEqual({
            encryptedMap: {key: 'secret'},
            'external-properties': {
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                },
                password: {
                    key: '',
                    kind: 'lookup',
                    provider: ''
                }
            },
            largePassword: ''
        });

        Form.toggleLookup('Password field');

        Form.expectModelToEqual({
            encryptedMap: {key: 'secret'},
            'external-properties': {
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                }
            },
            largePassword: '',
            password: ''
        });
        TestBindings.resetPristinity();
        Form.expectFieldsArePristine(['password', 'largePassword', 'vault', 'encryptedMap']);
    });

    it('should change field pristine state on model change, and not loose intermediate state after toggling', () => {
        Form.typeTextInLookupField('Password field', 'secret');
        Form.typeTextInLookupField('Large password field', 'secret2', true);
        Form.typeTextInToggledLookupField('Password field with lookup value', 'V', '1');

        const checkModel = (vault = {
            key: '1',
            kind: 'lookup',
            provider: 'V'
        }) => {
            Form.expectModelToEqual({
                encryptedMap: {key: 'secret'},
                'external-properties': {vault},
                largePassword: 'secret2',
                password: 'secret'
            });
        };

        checkModel();

        Form.toggleLookup('Password field');
        Form.toggleLookup('Password field');

        checkModel();

        TestBindings.resetPristinity();
        Form.typeTextInToggledLookupField('Password field with lookup value', 'S', '2');
        Form.expectedFieldsPristinity({
            password: true,
            largePassword: true,
            vault: false,
            encryptedMap: true
        });

        Form.toggleLookup('Password field with lookup value');
        Form.toggleLookup('Password field with lookup value');

        checkModel({
            key: '2',
            kind: 'lookup',
            provider: 'S'
        });

        Form.cleanValue('Password field');
        Form.cleanValue('Large password field');
        Form.toggleLookup('Password field with lookup value');

        TestBindings.expectObjectModelToEqual({
            encryptedMap: {key: 'secret'},
            largePassword: '',
            password: ''
        });
    });

    it('should display encrypted map', () => {
        Form.expectEncryptedMapHasEntry('Encrypted', 1, 'key', 'secret');
    });
});
