import React from 'react'; // eslint-disable-line

import {mount} from 'enzyme';

export default class ValidationsSpecHelper {
    constructor(componentSelector) {
        this.componentSelector = componentSelector;
    }

    buildFromWrapper = (wrapper) => {
        const clearButton = () => wrapper.find(`${this.componentSelector} .xl-react-btn-close`);
        const component = () => wrapper.find(`${this.componentSelector}`);
        const description = () => wrapper.find(`${this.componentSelector} .xl-react-components-description`);
        const errMsg = () => wrapper.find(`${this.componentSelector} .error`);
        const inputField = () => wrapper.find(`${this.componentSelector} input`);
        const changeInput = (value) => inputField().simulate('change', {target: {value}});
        const makeUnpristine = (originalValue = '', tempValue = '1') => {
            changeInput(tempValue);
            changeInput(originalValue);
            inputField().simulate('blur');
        };
        return {
            changeInput, clearButton, component, description, errMsg, inputField, makeUnpristine, wrapper
        };
    };

    buildFrom = (element) => this.buildFromWrapper(mount(element));

    shouldShowDefaultRequiredMsg = (element) => {
        it('should show a default input-field-required message after blur if no text is input and requiredMsg not set',
            () => {
                const {errMsg, makeUnpristine} = this.buildFrom(element);
                expect(errMsg()).not.toExist();
                makeUnpristine();
                expect(errMsg()).toExist();
                expect(errMsg()).toHaveText('This field is required');
            }
        );
    };

    shouldNotShowRequiredMsgIfStillPristine = (element) => {
        it('should show a default input-field-required message after blur if no text is input and requiredMsg not set',
            () => {
                const {errMsg, inputField} = this.buildFrom(element);
                expect(errMsg()).not.toExist();
                inputField().simulate('blur');
                expect(errMsg()).not.toExist();
            }
        );
    };

    shouldRemoveErrMsgWhenTyping = (element, typeText) => {
        it('should remove any error msg once the user starts typing', () => {
            const {changeInput, errMsg, makeUnpristine} = this.buildFrom(element);
            makeUnpristine();
            expect(errMsg()).toExist();
            changeInput(typeText);
            expect(errMsg()).not.toExist();
        });
    };

    shouldShowProvidedErrMsg = (element, expectedErrMsg) => {
        it('should show the provided requiredMsg on unpristine fields after blur if no text is input', () => {
            const {errMsg, makeUnpristine} = this.buildFrom(element);
            expect(errMsg()).not.toExist();
            makeUnpristine();
            expect(errMsg()).toExist();
            expect(errMsg()).toHaveText(expectedErrMsg);
        });
    };
}
