import React from 'react';
import {mount} from 'enzyme';
import moment from 'moment';

import Helper from '../../validations-spec-helper';
import {XlReactDipDatetime} from '~/app/xl-react-dip/xl-react-dip-datetime/xl-react-dip-datetime';
import {ERROR} from '~/app/utils/ci-utils';

describe('XlReactDipDatetime', () => {
    it('should be pristine if both the time and date components are', () => {
        const onPristinityChange = jest.fn();
        let api = {};

        function onRegister(datetimeApi) {
            api = datetimeApi;
        }

        const wrapper = mount(<XlReactDipDatetime
            model={moment('2017-04-14 07:15').toDate()}
            onPristinityChange={onPristinityChange}
            register={onRegister}
        />);

        expect(wrapper).toHaveState('pristine', true);
        const {makeUnpristine} = new Helper('.xl-react-dip-datetime .xl-react-widget-date').buildFromWrapper(wrapper);
        makeUnpristine('14/04/2016');
        expect(onPristinityChange).toBeCalledWith(false);
        api.resetPristinity();
        expect(onPristinityChange).toBeCalledWith(true);
        makeUnpristine('14/04/2017');
        expect(onPristinityChange).toBeCalledWith(false);
    });

    it('should honour custom validators', () => {
        const validator = (model) => ({
            condition: moment(model).hours() < 9 || moment(model).hours() >= 16,
            message: {show: true, text: 'Please specify a time during working hours', type: ERROR}
        });

        const date = moment('2017-04-14 07:15').toDate();

        const wrapper = mount(<XlReactDipDatetime model={date} validators={[validator]}/>);
        const {errMsg} = new Helper('.xl-react-dip-datetime').buildFromWrapper(wrapper);
        expect(errMsg()).toHaveText('Please specify a time during working hours');
    });
});
